//***************************************************************************
// Group Linslot / Linux - Slotrace Manager
// File arduino.hpp
// Date 16.11.08 - Jrg Wendel
// This code is distributed under the terms and conditions of the
// GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
//***************************************************************************

#ifndef _IO_ARDUINO_H_
#define _IO_ARDUINO_H_

//***************************************************************************
// Include
//***************************************************************************

#include <termios.h>

#include <QMutex>

#include <common.hpp>
#include <iointerface.hpp>

// default device: /dev/ttyUSB0

//***************************************************************************
// IO Interface
//***************************************************************************

class Arduino : public IoInterface, public SlotService
{
   public:

      enum Misc
      {
         sizeCmdMax = 100
      };

      enum Command
      {
         cmdUnknown = na,
         cmdDigitalInput,
         cmdAnalogInput,
         cmdBoardTime
      };

      // object

      Arduino();
      virtual ~Arduino();

      // interface

      virtual int open(const char* dev = 0);
      virtual int close();
      virtual int reopen(const char* dev = 0);
      virtual int isOpen()              { return fdDevice != 0; }
      virtual int flush();

      // read / write

      virtual int writeBit(int bit, int value);
      virtual int readOutBit(int bit);
      virtual int writeByte(unsigned char byte, int byteNum);
      virtual int readByte(unsigned char* byte, int byteNum);
      virtual int lookByte(int byteNum, unsigned  char* byte);

      // settings

      virtual int setTimeout(unsigned int timeout);
      virtual int setWriteTimeout(unsigned int timeout);

      timeval getLastIoTime() { return tvLastIoTime; }

   protected:

      const char* readCmd(char* command, char lookahed = 0);
      int dispatch(char* buf, IoValue& value);

      // data

      int fdDevice;
      int readTimeout;
      int writeTimeout;
      timeval tvBoarStartTime;
      timeval tvLastIoTime;
      unsigned char out[2];
      QMutex deviceMutex;
      struct termios oldtio;
};

//***************************************************************************
#endif // _IO_INTERFACE_H_
