//***************************************************************************
// Group Common Stuff
// File common.hpp
// Date 04.11.06 - Jrg Wendel
// This code is distributed under the terms and conditions of the
// GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
//***************************************************************************

#ifndef __COMMON_HPP__
#define __COMMON_HPP__

#include <sys/time.h>

//***************************************************************************
// Misc
//***************************************************************************

enum Result
{
   success = 0,
   fail = -1,
   na = -1,
   done = 0,
   ignore = -1,

   TB = 1
};

enum Eloquence
{
   eloOff,                   // 0
   eloAlways,                // 1
   eloDetail,                // 2
   eloDebug,                 // 3
   eloDebug1 = eloDebug,     // 3
   eloDebug2,                // 4
   eloDebug3                 // 5
};

extern int theEloquence;

int tell(int eloquence, const char* format, ...);
int tell(const char* format, ...);

const char* notNull(const char* s);

//***************************************************************************
// Slot Service
//***************************************************************************

class SlotService
{
   public:

      // declarations

      struct IoEvent
      {
         unsigned char byte;
         timeval tp;
      };

      // convert

      static int isBit(unsigned char byte, int bit);
      static void toggleBit(unsigned char &byte, int bit);
      static void setBitTo(unsigned char &byte, int bit, int value);

      static const char* toBinStr(unsigned char byte, char* mask);
      static double kmh(double meter, double usec);

      void tvNull(timeval* tv);
      timeval* tvNow(timeval* tv);
      timeval addMs2Tv(timeval tv, unsigned long milli);

      static long elapsed(const timeval* tpFrom, const timeval* tpNow);
      static const char* secStr(long usec, char* buf);
};

//***************************************************************************
#endif // __COMMON_HPP__
