//***************************************************************************
// Group Linslot / Linux - Slotrace Manager
// File setup.cc
// Date 16.12.06 - Jrg Wendel
// This code is distributed under the terms and conditions of the
// GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
//***************************************************************************

//***************************************************************************
// Include
//***************************************************************************

#include <QMessageBox>

#include <common.hpp>
#include <setup.hpp>

//***************************************************************************
// Object
//***************************************************************************

SetupDialog::SetupDialog(QString configPath)
{ 
   int count;
   char tmpDevice[100];

   setupUi(this);

   QCoreApplication::setOrganizationName("Jrg Wendel");
   QCoreApplication::setApplicationName("linslot") ;

   settings = new QSettings(configPath + "/linslotrc", QSettings::IniFormat);
 
   settings->beginGroup("common");

   roundCount = settings->value("roundCount", 10).toInt();
   roundLimit = settings->value("roundLimit", 50).toInt();
   slotLength = settings->value("slotLength", 14.0).toDouble();
   maxTimeRoundRace = settings->value("maxTimeRoundRace", 0).toInt();
   maxTimeTraining = settings->value("maxTimeTraining", 0).toInt();
   speedFactor = settings->value("speedFactor", 1).toInt();
   abortAtJumpTheGun = settings->value("abortAtJumpTheGun", 0).toInt();
   strcpy(tmpDevice, settings->value("usbDevice", "/dev/usb/iowarrior").toString().toAscii());
   strcpy(courseName, settings->value("course", "Heimstrecke").toString().toAscii());
   strcpy(databaseName, settings->value("databaseName", "linslot.db").toString().toAscii());
   penaltyAtJumpTheGun = settings->value("penaltyAtJumpTheGun", 5).toInt();

   settings->endGroup();

   settings->beginGroup("driver");
   count = settings->beginReadArray("driver");

   for (int i = 0; i < count; ++i) 
   {
      settings->setArrayIndex(i);
      listWidgetDriver->addItem(settings->value("name").toString());
   }

   settings->endArray();
   settings->endGroup();

   tabWidget->setCurrentIndex(0);

   // todo other devices ...

   comboBoxDevice->addItem("/dev/ttyUSB0");
   comboBoxDevice->addItem("/dev/usb/iowarrior");
   comboBoxDevice->setCurrentIndex(comboBoxDevice->findText(tmpDevice));

   lineEditCourse->setText(courseName);
   lineEditDbName->setText(databaseName);
   doubleSpinBoxSlotLength->setValue(slotLength);
   spinBoxRoundCount->setValue(roundCount);
   spinBoxTrainingRoundLimit->setValue(roundLimit);
   spinBoxJumpTheGunPenaltyTime->setValue(penaltyAtJumpTheGun);

   if (abortAtJumpTheGun)
      radioButtonJumpTheGunAbort->setChecked(true);
   else
      radioButtonJumpTheGunPenalty->setChecked(true);
      
   switch (speedFactor)
   {
      case 1:  radioButtonFaktor1->setChecked(true);   break;
      case 32: radioButtonFaktor32->setChecked(true);  break;
      case 24: radioButtonFaktor24->setChecked(true);  break;
      default: radioButtonFaktor1->setChecked(true);   break;
   }
}

SetupDialog::~SetupDialog()
{
   tell(eloAlways, "save settings");

   settings->beginGroup("common");

   settings->setValue("maxTimeTraining", maxTimeTraining);
   settings->setValue("maxTimeRoundRace", maxTimeRoundRace);
   settings->setValue("speedFactor", speedFactor);
   settings->setValue("slotLength", slotLength);
   settings->setValue("roundCount", roundCount);
   settings->setValue("roundLimit", roundLimit);
   settings->setValue("usbDevice", usbDevice);
   settings->setValue("course", courseName);
   settings->setValue("databaseName", databaseName);
   settings->setValue("abortAtJumpTheGun", abortAtJumpTheGun);
   settings->setValue("penaltyAtJumpTheGun", penaltyAtJumpTheGun);

   settings->endGroup();

   settings->beginGroup("driver");
   settings->beginWriteArray("driver");

   for (int i = 0; i < listWidgetDriver->count(); ++i) 
   {
      settings->setArrayIndex(i);
      settings->setValue("name", listWidgetDriver->item(i)->text());
   }

   settings->endArray();
   settings->endGroup();

   delete settings;
}

//***************************************************************************
// On Button Add Driver
//***************************************************************************

void SetupDialog::on_pushButtonAddDriver_clicked()
{
   if (!lineEditDriver->text().isEmpty())
   {
      QListWidgetItem* listItem;

      for (int i = 0; i < listWidgetDriver->count(); i++)
      {
         if ((listItem = listWidgetDriver->item(i)))
            if (listItem->text().compare(lineEditDriver->text()) == 0)
            {
               QMessageBox::information(this, "Info", "Der Fahrer ist bereits in der Liste enthalten!");
               return ;
            }
      }

      listWidgetDriver->addItem(lineEditDriver->text());
      lineEditDriver->setText("");
      lineEditDriver->setFocus();
   }
}

//***************************************************************************
// On Button Del Driver
//***************************************************************************

void SetupDialog::on_pushButtonDelDriver_clicked()
{
   if (listWidgetDriver->currentRow() >= 0)
      delete listWidgetDriver->takeItem(listWidgetDriver->currentRow());
}

//***************************************************************************
// On Round Limit Change
//***************************************************************************

void SetupDialog::on_spinBoxTrainingRoundLimit_valueChanged(int value)
{
   roundLimit = value;
}

//***************************************************************************
// On Round Count Change
//***************************************************************************

void SetupDialog::on_spinBoxRoundCount_valueChanged(int value)
{
   roundCount = value;
}

//***************************************************************************
// On Slot Lenght Change
//***************************************************************************

void SetupDialog::on_doubleSpinBoxSlotLength_valueChanged(double value)
{
   slotLength = value;
}

//***************************************************************************
// On Usb Device Editing Finished
//***************************************************************************

void SetupDialog::on_comboBoxDevice_currentIndexChanged(const QString text)
{
   strcpy(usbDevice, text.toAscii());
   tell(eloAlways, "set device to '%s'", usbDevice);
}

void SetupDialog::on_comboBoxDevice_editTextChanged(const QString text)
{
   strcpy(usbDevice, text.toAscii());
   tell(eloAlways, "set device to '%s'", usbDevice);
}

//***************************************************************************
// On Course Name Editing Finished
//***************************************************************************

void SetupDialog::on_lineEditCourse_editingFinished()
{
   strcpy(courseName, lineEditCourse->text().toAscii());
}

//***************************************************************************
// On Database Name Editing Finished
//***************************************************************************

void SetupDialog::on_lineEditDbName_editingFinished()
{
   strcpy(databaseName, lineEditDbName->text().toAscii());
}

//***************************************************************************
// On Round Race Editing Finished
//***************************************************************************

void SetupDialog::on_timeEditRoundRace_editingFinished()
{
   maxTimeRoundRace = 
      timeEditRoundRace->time().hour()*3600 
      + timeEditRoundRace->time().minute()*60 
      + timeEditRoundRace->time().second();
}

//***************************************************************************
// On Training Editing Finished
//***************************************************************************

void SetupDialog::on_timeEditTraining_editingFinished()
{
   maxTimeTraining = 
      timeEditTraining->time().hour()*3600 
      + timeEditTraining->time().minute()*60 
      + timeEditTraining->time().second();
}

//***************************************************************************
// On Tab Widget Changed
//***************************************************************************

void SetupDialog::on_tabWidget_currentChanged(int tab)
{
   if (tab == 1)
      lineEditDriver->setFocus();
}

//***************************************************************************
// Speed Factor
//***************************************************************************

void SetupDialog::on_radioButtonFaktor1_toggled(bool checked)
{
   if (checked) speedFactor = 1;
}

void SetupDialog::on_radioButtonFaktor32_toggled(bool checked)
{
   if (checked) speedFactor = 32;
}

void SetupDialog::on_radioButtonFaktor24_toggled(bool checked)
{
   if (checked) speedFactor = 24;
}

//***************************************************************************
// Jump The Gun
//***************************************************************************

void SetupDialog::on_radioButtonJumpTheGunAbort_toggled(bool checked)
{
   if (checked) abortAtJumpTheGun = true;
}

void SetupDialog::on_radioButtonJumpTheGunPenalty_toggled(bool checked)
{
   if (checked) abortAtJumpTheGun = false;
}

void SetupDialog::on_spinBoxJumpTheGunPenaltyTime_valueChanged(int value)
{
   penaltyAtJumpTheGun = value;
}
